function HuskPlayerMovement:_unregister_cop_revive()	
	if self._revive_cop_so_id then
		managers.groupai:state():remove_special_objective(self._revive_cop_so_id)

		self._revive_cop_so_id = nil
	elseif self._revive_cop then
		local rescuer = self._revive_cop
		self._revive_cop = nil
		if rescuer and alive(rescuer) then
			rescuer:brain():set_objective()
		end	
	end
end

function HuskPlayerMovement:on_cop_revive_started(revive_timer, cop)
	if self._revive_cop ~= cop then
		return
	end
	
	if self._unit and alive(self._unit) then
		if self._unit:interaction() and self._unit:interaction():active() then
			self._unit:interaction():_at_interact_start(cop, revive_timer)
		end
	end
	
	if self._revive_cop_so_id then
		managers.groupai:state():remove_special_objective(self._revive_cop_so_id)

		self._revive_cop_so_id = nil
	end
end

function HuskPlayerMovement:on_cop_revive_failed(cop)
	if self._revive_cop ~= cop then
		return
	end
	if self._unit and alive(self._unit) then
		if self._unit:interaction() and self._unit:interaction():active() then
			self._unit:interaction():interact_interupt(cop)
		end
	end
	
	self._revive_cop = nil
	self:_register_cop_revive()
end

function HuskPlayerMovement:on_cop_revive_completed(cop)
	if self._revive_cop ~= cop then
		return
	end
	
	if self._unit and alive(self._unit) then
		if self._unit:interaction() and self._unit:interaction():active() then
			self._unit:interaction():clean_interact(self._revive_cop)
		end
	end
	
	self._revive_cop = nil
	self:_unregister_cop_revive()
end

function HuskPlayerMovement:on_cop_rescue_SO_administered(cop)
	self._revive_cop = cop
	self._revive_cop_so_id = nil
end

function HuskPlayerMovement:_register_cop_revive()
	if self._revive_cop_so_id or not managers.navigation:is_data_ready() then
		return
	end
	local revive_timer = tweak_data.interaction.revive.timer
	local pos, rot = PlayerBleedOut:get_revive_pos(self._unit)
	local followup_objective = {
		interrupt_health = 1,
		interrupt_dis = -1,
		type = "act",
		action = {
			sync = true,
			body_part = 1,
			type = "idle"
		}
	}
	local objective = {
		type = "act",
		haste = "run",
		destroy_clbk_key = false,
		stance = "hos",
		scan = true,
		follow_unit = self._unit,
		nav_seg = self._unit:movement():nav_tracker():nav_segment(),
		pos = pos,
		rot = rot,
		fail_clbk = callback(self, self, "on_cop_revive_failed"),
		complete_clbk = callback(self, self, "on_cop_revive_completed"),
		action_start_clbk = callback(self, self, "on_cop_revive_started", revive_timer),
		action = {
			align_sync = true,
			type = "act",
			body_part = 1,
			variant = "revive",
			blocks = {
				heavy_hurt = -1,
				hurt = -1,
				action = -1,
				stand = -1,
				light_hurt = -1,
				aim = -1,
				crouch = -1,
				reload = -1,
				walk = -1
			}
		},
		action_duration = revive_timer,
		followup_objective = followup_objective
	}
	local so_descriptor = {
		interval = 1,
		AI_group = "enemies",
		base_chance = 1,
		chance_inc = 0,
		access = PlayerBleedOut:enemy_access(),
		usage_amount = 1,
		objective = objective,
		search_pos = mvector3.copy(self._unit:movement():m_head_pos()),
		admin_clbk = callback(self, self, "on_cop_rescue_SO_administered"),
		verification_clbk = callback(PlayerBleedOut, PlayerBleedOut, "cop_rescue_SO_verification")
	}
	local so_id = "HuskPlayerCopRevive"..tostring(self._unit:key())
	self._revive_cop_so_id = so_id

	managers.groupai:state():add_special_objective(so_id, so_descriptor)
end


local _HuskPlayerMovement_set_need_revive = HuskPlayerMovement.set_need_revive
function HuskPlayerMovement:set_need_revive(need_revive, ...)
	if self._need_revive ~= need_revive then
		if Network:is_server() then
			if self._state ~= "arrested" then
				if need_revive and not self._revive_cop_so_id and not self._revive_cop then
					self:_register_cop_revive()
				elseif not need_revive and (self._revive_cop_so_id or self._revive_cop) then
					self:_unregister_cop_revive()
				end
			end
		end
	end
	_HuskPlayerMovement_set_need_revive(self, need_revive, ...)
end
