local _PlayerBleedOut_enter = PlayerBleedOut.enter
local _PlayerBleedOut_exit = PlayerBleedOut.exit


function PlayerBleedOut:enter(state_data, enter_data)
	_PlayerBleedOut_enter(self, state_data, enter_data)
	
	self._revive_SO_data_cop = {unit = self._unit}
	
	if Network:is_server() and self._ext_movement:nav_tracker() then
		PlayerBleedOut:_register_cop_revive(self._revive_SO_data_cop)
	end
	
end


function PlayerBleedOut:exit(state_data, new_state_name)
	local exit_data = _PlayerBleedOut_exit(self, state_data, new_state_name)	
	if new_state_name == "fatal" then
		exit_data.revive_SO_data_cop = self._revive_SO_data_cop
		self._revive_SO_data_cop = nil
	else
		if Network:is_server() then
			self:_unregister_cop_revive()
		end
	end
	return exit_data
end

function PlayerBleedOut:on_revived_boosts(reviving_unit, revive_unit)
end

function PlayerBleedOut:get_all_variations_of_access()
	local accesses = {}
	for _, char_tweak in pairs(tweak_data.character) do
		if char_tweak and type(char_tweak) == "table" and char_tweak.access then
			if table.contains(accesses, char_tweak.access) then
				table.insert(accesses, char_tweak.access)
			end
		end	
	end
	return accesses
end

function PlayerBleedOut:get_revive_pos(follow_unit)
	local is_local_player = follow_unit:base().is_local_player
	local revive_u_mv = follow_unit:movement()
	local revive_u_tracker = revive_u_mv:nav_tracker()
	local revive_u_rot = is_local_player and Rotation(0, 0, 0) or revive_u_mv:m_rot()
	local revive_u_fwd = revive_u_rot:y()
	local revive_u_right = revive_u_rot:x()
	local revive_u_pos = revive_u_tracker:lost() and revive_u_tracker:field_position() or revive_u_mv:m_pos()
	local ray_params = {
		trace = true,
		tracker_from = revive_u_tracker
	}

	if revive_u_tracker:lost() then
		ray_params.pos_from = revive_u_pos
	end

	local stand_dis = nil

	if is_local_player or follow_unit:base().is_husk_player then
		stand_dis = 120
	else
		stand_dis = 90
		local mid_pos = mvector3.copy(revive_u_fwd)

		mvector3.multiply(mid_pos, -20)
		mvector3.add(mid_pos, revive_u_pos)

		ray_params.pos_to = mid_pos
		local ray_res = managers.navigation:raycast(ray_params)
		revive_u_pos = ray_params.trace[1]
	end

	local rand_side_mul = math.random() > 0.5 and 1 or -1
	local revive_pos = mvector3.copy(revive_u_right)

	mvector3.multiply(revive_pos, rand_side_mul * stand_dis)
	mvector3.add(revive_pos, revive_u_pos)

	ray_params.pos_to = revive_pos
	local ray_res = managers.navigation:raycast(ray_params)

	if ray_res then
		local opposite_pos = mvector3.copy(revive_u_right)

		mvector3.multiply(opposite_pos, -rand_side_mul * stand_dis)
		mvector3.add(opposite_pos, revive_u_pos)

		ray_params.pos_to = opposite_pos
		local old_trace = ray_params.trace[1]
		local opposite_ray_res = managers.navigation:raycast(ray_params)

		if opposite_ray_res then
			if mvector3.distance(revive_pos, revive_u_pos) < mvector3.distance(ray_params.trace[1], revive_u_pos) then
				revive_pos = ray_params.trace[1]
			else
				revive_pos = old_trace
			end
		else
			revive_pos = ray_params.trace[1]
		end
	else
		revive_pos = ray_params.trace[1]
	end

	local revive_rot = revive_u_pos - revive_pos
	local revive_rot = Rotation(revive_rot, math.UP)
	return revive_pos, revive_rot
end

function PlayerBleedOut:on_cop_revive_completed(revive_SO_data_cop, cop)
	if not revive_SO_data_cop then
		return
	end
	
	if cop ~= revive_SO_data_cop.my_cop then
		return
	end

	if revive_SO_data_cop.my_cop and alive(revive_SO_data_cop.my_cop) then
		if revive_SO_data_cop.unit and alive(revive_SO_data_cop.unit) and revive_SO_data_cop.unit.character_damage and revive_SO_data_cop.unit:character_damage() and revive_SO_data_cop.unit:character_damage():need_revive()  then
			revive_SO_data_cop.unit:character_damage():revive()
		end
	end
	self:_unregister_cop_revive()
end

function PlayerBleedOut:on_cop_revive_failed(revive_SO_data_cop, cop)
	if not revive_SO_data_cop then
		return
	end

	if revive_SO_data_cop.my_cop then
		if cop ~= revive_SO_data_cop.my_cop then
			return
		end

		local revive_unit = revive_SO_data_cop.unit
		if revive_unit and alive(revive_unit) and revive_unit:character_damage() and revive_unit:character_damage():need_revive() then
			revive_unit:character_damage():unpause_downed_timer()
		end
		
		revive_SO_data_cop.my_cop = nil
		self:_register_cop_revive(revive_SO_data_cop)
	end
end

function PlayerBleedOut:on_cop_revive_started(revive_SO_data_cop, cop)	
	if not revive_SO_data_cop then
		return
	end
	
	if cop ~= revive_SO_data_cop.my_cop then
		return
	end
	
	local revive_unit = revive_SO_data_cop.unit
	if revive_unit and alive(revive_unit) and revive_unit:character_damage() and revive_unit:character_damage():need_revive()  then
		revive_unit:character_damage():pause_downed_timer()
	end

	if revive_SO_data_cop.cop_SO_id then
		managers.groupai:state():remove_special_objective(revive_SO_data_cop.cop_SO_id)

		revive_SO_data_cop.cop_SO_id = nil
	end
end

function PlayerBleedOut:enemy_access()
	local access = managers.navigation:convert_access_filter_to_number({
		"teamAI1", --- Better Bots has a code where it will change the "access" to teamAI1. Include this to make it so converted enemies with that access can be included.
		"gangster",
		"security",
		"security_patrol",
		"cop",
		"fbi",
		"swat",
		"murky",
		"sniper",
		"spooc",
		"tank",
		"taser"
	})
	return access
end

function PlayerBleedOut:cop_rescue_SO_verification(cop)
	local u_data = managers.groupai:state()._police[cop:key()]

	if not u_data then
		return false
	end
	
	return u_data.is_converted and true or false
end

function PlayerBleedOut:on_cop_rescue_SO_administered(revive_SO_data_cop, receiver_unit)
	revive_SO_data_cop.my_cop = receiver_unit
	revive_SO_data_cop.cop_SO_id = nil
end


function PlayerBleedOut:_unregister_cop_revive()
	if not self._revive_SO_data_cop then
		return
	end

	if self._revive_SO_data_cop.cop_SO_id then
		managers.groupai:state():remove_special_objective(self._revive_SO_data_cop.cop_SO_id)

		self._cop_SO_id = nil
	elseif self._revive_SO_data_cop.my_cop then
		local rescuer = self._revive_SO_data_cop.my_cop
		self._revive_SO_data_cop.my_cop = nil

		if rescuer and alive(rescuer) then
			rescuer:brain():set_objective()
		end
	end
end

function PlayerBleedOut:_register_cop_revive(revive_SO_data_cop)
	if revive_SO_data_cop.cop_SO_id or not managers.navigation:is_data_ready() then
		return
	end
	local pos, rot = PlayerBleedOut:get_revive_pos(revive_SO_data_cop.unit)
	local followup_objective = {
		interrupt_health = 1,
		interrupt_dis = -1,
		type = "act",
		action = {
			sync = true,
			body_part = 1,
			type = "idle"
		}
	}
	local objective = {
		type = "act",
		haste = "run",
		destroy_clbk_key = false,
		stance = "hos",
		scan = true,
		follow_unit = revive_SO_data_cop.unit,
		nav_seg = revive_SO_data_cop.unit:movement():nav_tracker():nav_segment(),
		pos = pos,
		rot = rot,
		fail_clbk = callback(PlayerBleedOut, PlayerBleedOut, "on_cop_revive_failed", revive_SO_data_cop),
		complete_clbk = callback(PlayerBleedOut, PlayerBleedOut, "on_cop_revive_completed", revive_SO_data_cop),
		action_start_clbk = callback(PlayerBleedOut, PlayerBleedOut, "on_cop_revive_started", revive_SO_data_cop),
		action = {
			align_sync = true,
			type = "act",
			body_part = 1,
			variant = "revive",
			blocks = {
				heavy_hurt = -1,
				hurt = -1,
				action = -1,
				stand = -1,
				reload = -1,
				light_hurt = -1,
				aim = -1,
				crouch = -1,
				walk = -1
			}
		},
		action_duration = tweak_data.interaction.revive.timer,
		followup_objective = followup_objective
	}
	local so_descriptor = {
		interval = 1,
		AI_group = "enemies",
		base_chance = 1,
		chance_inc = 0,
		access = PlayerBleedOut:enemy_access(),
		usage_amount = 1,
		objective = objective,
		search_pos = mvector3.copy(revive_SO_data_cop.unit:movement():m_head_pos()),
		admin_clbk = callback(PlayerBleedOut, PlayerBleedOut, "on_cop_rescue_SO_administered", revive_SO_data_cop),
		verification_clbk = callback(PlayerBleedOut, PlayerBleedOut, "cop_rescue_SO_verification")
	}
	local so_id = "PlayerCopRevive"
	revive_SO_data_cop.cop_SO_id = so_id

	managers.groupai:state():add_special_objective(so_id, so_descriptor)
end
