local is_offline = Global and Global.game_settings and Global.game_settings.single_player


local _TeamAILogicDisabled_enter = TeamAILogicDisabled.enter
function TeamAILogicDisabled.enter(data, new_logic_name, enter_params)
	_TeamAILogicDisabled_enter(data, new_logic_name, enter_params)
	local my_data = data.internal_data
	if Network:is_server() and data.unit:character_damage():need_revive() then
		if data.name ~= "surrender" then
			TeamAILogicDisabled._register_cop_revive(data, my_data)
		end
	end
	
end

local _TeamAILogicDisabled_exit = TeamAILogicDisabled.exit
function TeamAILogicDisabled.exit(data, new_logic_name, enter_params)
	local my_data = data.internal_data
	if Network:is_server() and new_logic_name ~= "inactive" then
		TeamAILogicDisabled._unregister_cop_revive(data)
	end
	_TeamAILogicDisabled_exit(data, new_logic_name, enter_params)
end

function TeamAILogicDisabled._unregister_cop_revive(data)
	local my_data = data.internal_data
	if my_data then
		if my_data.cop_so_id then
			managers.groupai:state():remove_special_objective(my_data.cop_so_id)

			my_data.cop_so_id = nil
		elseif my_data.cop_rescuer then
			local rescuer = my_data.cop_rescuer
			my_data.cop_rescuer = nil

			if alive(rescuer) then
				rescuer:brain():set_objective(nil)
			end
		end
	end
end

function TeamAILogicDisabled:on_cop_revive_failed(data, cop)
	local my_data = data.internal_data
	if my_data and my_data.cop_rescuer then
		if cop ~= my_data.cop_rescuer then
			return
		end

		if data.unit:interaction() then
			data.unit:interaction():interact_interupt(cop)
		end

		my_data.cop_rescuer = nil
		TeamAILogicDisabled._register_cop_revive(data, my_data)
	end
end

function TeamAILogicDisabled:on_cop_revive_completed(data, cop)
	local my_data = data.internal_data
	if not my_data or cop ~= my_data.cop_rescuer then
		return
	end
	
	if data.unit and alive(data.unit) then
		if data.unit:interaction() and data.unit:interaction():active() then
			data.unit:interaction():clean_interact(my_data.cop_rescuer)
		end
	end
	
	TeamAILogicDisabled._unregister_cop_revive(data)
end

function TeamAILogicDisabled:on_cop_revive_started(data, cop)
	local my_data = data.internal_data
	if cop ~= my_data.cop_rescuer then
		return
	end
	
	if data.unit:interaction() then
		if data.unit:interaction():active() then
			data.unit:interaction():interact_start(cop)
		end
	end

	if my_data.cop_so_id then
		managers.groupai:state():remove_special_objective(my_data.cop_so_id)

		my_data.cop_so_id = nil
	end
end


function TeamAILogicDisabled:on_cop_rescue_SO_administered(data, receiver_unit)
	local my_data = data.internal_data
	if not my_data then
		return
	end
	my_data.cop_rescuer = receiver_unit
	my_data.cop_so_id = nil
end

function TeamAILogicDisabled._register_cop_revive(data, my_data)
	if my_data.cop_so_id or not managers.navigation:is_data_ready() then
		return
	end
	local pos, rot = PlayerBleedOut:get_revive_pos(data.unit)
	local followup_objective = {
		interrupt_health = 1,
		interrupt_dis = -1,
		type = "act",
		action = {
			sync = true,
			body_part = 1,
			type = "idle"
		}
	}
	local objective = {
		type = "act",
		haste = "run",
		follow_unit = data.unit,
		nav_seg = data.unit:movement():nav_tracker():nav_segment(),
		pos = pos,
		rot = rot,
		destroy_clbk_key = false,
		stance = "hos",
		scan = true,
		fail_clbk = callback(TeamAILogicDisabled, TeamAILogicDisabled, "on_cop_revive_failed", data),
		complete_clbk = callback(TeamAILogicDisabled, TeamAILogicDisabled, "on_cop_revive_completed", data),
		action_start_clbk = callback(TeamAILogicDisabled, TeamAILogicDisabled, "on_cop_revive_started", data),
		action = {
			align_sync = true,
			type = "act",
			body_part = 1,
			variant = "revive",
			blocks = {
				heavy_hurt = -1,
				hurt = -1,
				action = -1,
				stand = -1,
				reload = -1,
				light_hurt = -1,
				aim = -1,
				crouch = -1,
				walk = -1
			}
		},
		action_duration = tweak_data.interaction.revive.timer,
		followup_objective = followup_objective
	}
	local so_descriptor = {
		interval = 1,
		AI_group = "enemies",
		base_chance = 1,
		chance_inc = 0,
		access = PlayerBleedOut:enemy_access(),
		usage_amount = 1,
		objective = objective,
		search_pos = mvector3.copy(data.m_pos),
		admin_clbk = callback(TeamAILogicDisabled, TeamAILogicDisabled, "on_cop_rescue_SO_administered", data),
		verification_clbk = callback(PlayerBleedOut, PlayerBleedOut, "cop_rescue_SO_verification")
	}
	local so_id = "TeamAICopRevive"..tostring(data.key)
	my_data.cop_SO_id = so_id

	managers.groupai:state():add_special_objective(so_id, so_descriptor)
end